# Bach mouse mammary gland (10X Genomics)

<script>
document.addEventListener("click", function (event) {
    if (event.target.classList.contains("rebook-collapse")) {
        event.target.classList.toggle("active");
        var content = event.target.nextElementSibling;
        if (content.style.display === "block") {
            content.style.display = "none";
        } else {
            content.style.display = "block";
        }
    }
})
</script>

<style>
.rebook-collapse {
  background-color: #eee;
  color: #444;
  cursor: pointer;
  padding: 18px;
  width: 100%;
  border: none;
  text-align: left;
  outline: none;
  font-size: 15px;
}

.rebook-content {
  padding: 0 18px;
  display: none;
  overflow: hidden;
  background-color: #f1f1f1;
}
</style>

## Introduction

This performs an analysis of the @bach2017differentiation 10X Genomics dataset,
from which we will consider a single sample of epithelial cells from the mouse mammary gland during gestation.

## Data loading


```r
library(scRNAseq)
sce.mam <- BachMammaryData(samples="G_1")
```


```r
library(scater)
rownames(sce.mam) <- uniquifyFeatureNames(
    rowData(sce.mam)$Ensembl, rowData(sce.mam)$Symbol)

library(AnnotationHub)
ens.mm.v97 <- AnnotationHub()[["AH73905"]]
rowData(sce.mam)$SEQNAME <- mapIds(ens.mm.v97, keys=rowData(sce.mam)$Ensembl,
    keytype="GENEID", column="SEQNAME")
```

## Quality control


```r
unfiltered <- sce.mam
```


```r
is.mito <- rowData(sce.mam)$SEQNAME == "MT"
stats <- perCellQCMetrics(sce.mam, subsets=list(Mito=which(is.mito)))
qc <- quickPerCellQC(stats, percent_subsets="subsets_Mito_percent")
sce.mam <- sce.mam[,!qc$discard]
```


```r
colData(unfiltered) <- cbind(colData(unfiltered), stats)
unfiltered$discard <- qc$discard

gridExtra::grid.arrange(
    plotColData(unfiltered, y="sum", colour_by="discard") + 
        scale_y_log10() + ggtitle("Total count"),
    plotColData(unfiltered, y="detected", colour_by="discard") + 
        scale_y_log10() + ggtitle("Detected features"),
    plotColData(unfiltered, y="subsets_Mito_percent", 
        colour_by="discard") + ggtitle("Mito percent"),
    ncol=2
)
```

<div class="figure">
<img src="bach-mammary_files/figure-html/unref-bach-qc-dist-1.png" alt="Distribution of each QC metric across cells in the Bach mammary gland dataset. Each point represents a cell and is colored according to whether that cell was discarded." width="672" />
<p class="caption">(\#fig:unref-bach-qc-dist)Distribution of each QC metric across cells in the Bach mammary gland dataset. Each point represents a cell and is colored according to whether that cell was discarded.</p>
</div>


```r
plotColData(unfiltered, x="sum", y="subsets_Mito_percent", 
    colour_by="discard") + scale_x_log10()
```

<div class="figure">
<img src="bach-mammary_files/figure-html/unref-bach-qc-comp-1.png" alt="Percentage of mitochondrial reads in each cell in the Bach mammary gland dataset compared to its total count. Each point represents a cell and is colored according to whether that cell was discarded." width="672" />
<p class="caption">(\#fig:unref-bach-qc-comp)Percentage of mitochondrial reads in each cell in the Bach mammary gland dataset compared to its total count. Each point represents a cell and is colored according to whether that cell was discarded.</p>
</div>


```r
colSums(as.matrix(qc))
```

```
##              low_lib_size            low_n_features high_subsets_Mito_percent 
##                         0                         0                       143 
##                   discard 
##                       143
```

## Normalization 


```r
library(scran)
set.seed(101000110)
clusters <- quickCluster(sce.mam)
sce.mam <- computeSumFactors(sce.mam, clusters=clusters)
sce.mam <- logNormCounts(sce.mam)
```


```r
summary(sizeFactors(sce.mam))
```

```
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##   0.271   0.522   0.758   1.000   1.204  10.958
```


```r
plot(librarySizeFactors(sce.mam), sizeFactors(sce.mam), pch=16,
    xlab="Library size factors", ylab="Deconvolution factors", log="xy")
```

<div class="figure">
<img src="bach-mammary_files/figure-html/unref-bach-norm-1.png" alt="Relationship between the library size factors and the deconvolution size factors in the Bach mammary gland dataset." width="672" />
<p class="caption">(\#fig:unref-bach-norm)Relationship between the library size factors and the deconvolution size factors in the Bach mammary gland dataset.</p>
</div>

## Variance modelling

We use a Poisson-based technical trend to capture more genuine biological variation in the biological component.


```r
set.seed(00010101)
dec.mam <- modelGeneVarByPoisson(sce.mam)
top.mam <- getTopHVGs(dec.mam, prop=0.1)
```


```r
plot(dec.mam$mean, dec.mam$total, pch=16, cex=0.5,
    xlab="Mean of log-expression", ylab="Variance of log-expression")
curfit <- metadata(dec.mam)
curve(curfit$trend(x), col='dodgerblue', add=TRUE, lwd=2)
```

<div class="figure">
<img src="bach-mammary_files/figure-html/unref-bach-var-1.png" alt="Per-gene variance as a function of the mean for the log-expression values in the Bach mammary gland dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to simulated Poisson counts." width="672" />
<p class="caption">(\#fig:unref-bach-var)Per-gene variance as a function of the mean for the log-expression values in the Bach mammary gland dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to simulated Poisson counts.</p>
</div>

## Dimensionality reduction


```r
library(BiocSingular)
set.seed(101010011)
sce.mam <- denoisePCA(sce.mam, technical=dec.mam, subset.row=top.mam)
sce.mam <- runTSNE(sce.mam, dimred="PCA")
```


```r
ncol(reducedDim(sce.mam, "PCA"))
```

```
## [1] 15
```

## Clustering 

We use a higher `k` to obtain coarser clusters (for use in `doubletCluster()` later).


```r
snn.gr <- buildSNNGraph(sce.mam, use.dimred="PCA", k=25)
colLabels(sce.mam) <- factor(igraph::cluster_walktrap(snn.gr)$membership)
```


```r
table(colLabels(sce.mam))
```

```
## 
##   1   2   3   4   5   6   7   8   9  10 
## 550 799 716 452  24  84  52  39  32  24
```


```r
plotTSNE(sce.mam, colour_by="label")
```

<div class="figure">
<img src="bach-mammary_files/figure-html/unref-bach-tsne-1.png" alt="Obligatory $t$-SNE plot of the Bach mammary gland dataset, where each point represents a cell and is colored according to the assigned cluster." width="672" />
<p class="caption">(\#fig:unref-bach-tsne)Obligatory $t$-SNE plot of the Bach mammary gland dataset, where each point represents a cell and is colored according to the assigned cluster.</p>
</div>

## Session Info {-}

<button class="rebook-collapse">View session info</button>
<div class="rebook-content">
```
R version 4.3.0 RC (2023-04-13 r84269)
Platform: x86_64-pc-linux-gnu (64-bit)
Running under: Ubuntu 22.04.2 LTS

Matrix products: default
BLAS:   /home/biocbuild/bbs-3.17-bioc/R/lib/libRblas.so 
LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.10.0

locale:
 [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
 [3] LC_TIME=en_GB              LC_COLLATE=C              
 [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
 [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
 [9] LC_ADDRESS=C               LC_TELEPHONE=C            
[11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       

time zone: America/New_York
tzcode source: system (glibc)

attached base packages:
[1] stats4    stats     graphics  grDevices utils     datasets  methods  
[8] base     

other attached packages:
 [1] BiocSingular_1.16.0         scran_1.28.0               
 [3] AnnotationHub_3.8.0         BiocFileCache_2.8.0        
 [5] dbplyr_2.3.2                scater_1.28.0              
 [7] ggplot2_3.4.2               scuttle_1.10.0             
 [9] ensembldb_2.24.0            AnnotationFilter_1.24.0    
[11] GenomicFeatures_1.52.0      AnnotationDbi_1.62.0       
[13] scRNAseq_2.13.0             SingleCellExperiment_1.22.0
[15] SummarizedExperiment_1.30.0 Biobase_2.60.0             
[17] GenomicRanges_1.52.0        GenomeInfoDb_1.36.0        
[19] IRanges_2.34.0              S4Vectors_0.38.0           
[21] BiocGenerics_0.46.0         MatrixGenerics_1.12.0      
[23] matrixStats_0.63.0          BiocStyle_2.28.0           
[25] rebook_1.10.0              

loaded via a namespace (and not attached):
  [1] jsonlite_1.8.4                CodeDepends_0.6.5            
  [3] magrittr_2.0.3                ggbeeswarm_0.7.1             
  [5] farver_2.1.1                  rmarkdown_2.21               
  [7] BiocIO_1.10.0                 zlibbioc_1.46.0              
  [9] vctrs_0.6.2                   memoise_2.0.1                
 [11] Rsamtools_2.16.0              DelayedMatrixStats_1.22.0    
 [13] RCurl_1.98-1.12               htmltools_0.5.5              
 [15] progress_1.2.2                curl_5.0.0                   
 [17] BiocNeighbors_1.18.0          sass_0.4.5                   
 [19] bslib_0.4.2                   cachem_1.0.7                 
 [21] GenomicAlignments_1.36.0      igraph_1.4.2                 
 [23] mime_0.12                     lifecycle_1.0.3              
 [25] pkgconfig_2.0.3               rsvd_1.0.5                   
 [27] Matrix_1.5-4                  R6_2.5.1                     
 [29] fastmap_1.1.1                 GenomeInfoDbData_1.2.10      
 [31] shiny_1.7.4                   digest_0.6.31                
 [33] colorspace_2.1-0              dqrng_0.3.0                  
 [35] irlba_2.3.5.1                 ExperimentHub_2.8.0          
 [37] RSQLite_2.3.1                 beachmat_2.16.0              
 [39] labeling_0.4.2                filelock_1.0.2               
 [41] fansi_1.0.4                   httr_1.4.5                   
 [43] compiler_4.3.0                bit64_4.0.5                  
 [45] withr_2.5.0                   BiocParallel_1.34.0          
 [47] viridis_0.6.2                 DBI_1.1.3                    
 [49] highr_0.10                    biomaRt_2.56.0               
 [51] rappdirs_0.3.3                DelayedArray_0.26.0          
 [53] bluster_1.10.0                rjson_0.2.21                 
 [55] tools_4.3.0                   vipor_0.4.5                  
 [57] beeswarm_0.4.0                interactiveDisplayBase_1.38.0
 [59] httpuv_1.6.9                  glue_1.6.2                   
 [61] restfulr_0.0.15               promises_1.2.0.1             
 [63] grid_4.3.0                    Rtsne_0.16                   
 [65] cluster_2.1.4                 generics_0.1.3               
 [67] gtable_0.3.3                  hms_1.1.3                    
 [69] metapod_1.8.0                 ScaledMatrix_1.8.0           
 [71] xml2_1.3.3                    utf8_1.2.3                   
 [73] XVector_0.40.0                ggrepel_0.9.3                
 [75] BiocVersion_3.17.1            pillar_1.9.0                 
 [77] stringr_1.5.0                 limma_3.56.0                 
 [79] later_1.3.0                   dplyr_1.1.2                  
 [81] lattice_0.21-8                rtracklayer_1.60.0           
 [83] bit_4.0.5                     tidyselect_1.2.0             
 [85] locfit_1.5-9.7                Biostrings_2.68.0            
 [87] knitr_1.42                    gridExtra_2.3                
 [89] bookdown_0.33                 ProtGenerics_1.32.0          
 [91] edgeR_3.42.0                  xfun_0.39                    
 [93] statmod_1.5.0                 stringi_1.7.12               
 [95] lazyeval_0.2.2                yaml_2.3.7                   
 [97] evaluate_0.20                 codetools_0.2-19             
 [99] tibble_3.2.1                  BiocManager_1.30.20          
[101] graph_1.78.0                  cli_3.6.1                    
[103] xtable_1.8-4                  munsell_0.5.0                
[105] jquerylib_0.1.4               Rcpp_1.0.10                  
[107] dir.expiry_1.8.0              png_0.1-8                    
[109] XML_3.99-0.14                 parallel_4.3.0               
[111] ellipsis_0.3.2                blob_1.2.4                   
[113] prettyunits_1.1.1             sparseMatrixStats_1.12.0     
[115] bitops_1.0-7                  viridisLite_0.4.1            
[117] scales_1.2.1                  purrr_1.0.1                  
[119] crayon_1.5.2                  rlang_1.1.0                  
[121] cowplot_1.1.1                 KEGGREST_1.40.0              
```
</div>
